All of lore.kernel.org
 help / color / mirror / Atom feed
* [PATCH] ioreq: don't wrongly claim "success" in ioreq_send_buffered()
@ 2024-09-11 12:19 Jan Beulich
  2024-09-16 21:27 ` Julien Grall
  0 siblings, 1 reply; 5+ messages in thread
From: Jan Beulich @ 2024-09-11 12:19 UTC (permalink / raw)
  To: xen-devel@lists.xenproject.org
  Cc: Julien Grall, Stefano Stabellini, Volodymyr Babchuk,
	Bertrand Marquis, Michal Orzel, Andrew Cooper,
	Roger Pau Monné

Returning a literal number is a bad idea anyway when all other returns
use IOREQ_STATUS_* values. While that's maybe intended on Arm (mapping
to IO_ABORT), mapping to X86EMUL_OKAY is surely wrong on x86.

Fixes: f6bf39f84f82 ("x86/hvm: add support for broadcast of buffered ioreqs...")
Signed-off-by: Jan Beulich <jbeulich@suse.com>
---
Judging from history, it may want to be IOREQ_STATUS_UNHANDLED instead,
eliminating the need for IOREQ_STATUS_BAD. That'll be a behavioral
change on Arm then too, though.

Shouldn't IOREQ_READ requests also be rejected here, for the result of
a read not possibly coming from anywhere, yet a (bogus) caller then
assuming some data was actually returned?

--- a/xen/arch/arm/include/asm/ioreq.h
+++ b/xen/arch/arm/include/asm/ioreq.h
@@ -56,6 +56,7 @@ static inline void msix_write_completion
 #define IOREQ_STATUS_HANDLED     IO_HANDLED
 #define IOREQ_STATUS_UNHANDLED   IO_UNHANDLED
 #define IOREQ_STATUS_RETRY       IO_RETRY
+#define IOREQ_STATUS_BAD         IO_ABORT
 
 #endif /* __ASM_ARM_IOREQ_H__ */
 
--- a/xen/arch/x86/include/asm/hvm/ioreq.h
+++ b/xen/arch/x86/include/asm/hvm/ioreq.h
@@ -12,6 +12,7 @@
 #define IOREQ_STATUS_HANDLED     X86EMUL_OKAY
 #define IOREQ_STATUS_UNHANDLED   X86EMUL_UNHANDLEABLE
 #define IOREQ_STATUS_RETRY       X86EMUL_RETRY
+#define IOREQ_STATUS_BAD         X86EMUL_UNRECOGNIZED
 
 #endif /* __ASM_X86_HVM_IOREQ_H__ */
 
--- a/xen/common/ioreq.c
+++ b/xen/common/ioreq.c
@@ -1175,7 +1175,7 @@ static int ioreq_send_buffered(struct io
         return IOREQ_STATUS_UNHANDLED;
 
     /*
      * Return 0 for the cases we can't deal with:
      *  - 'addr' is only a 20-bit field, so we cannot address beyond 1MB
      *  - we cannot buffer accesses to guest memory buffers, as the guest
      *    may expect the memory buffer to be synchronously accessed
@@ -1183,7 +1183,7 @@ static int ioreq_send_buffered(struct io
      *    support data_is_ptr we do not waste space for the count field either
      */
     if ( (p->addr > 0xfffffUL) || p->data_is_ptr || (p->count != 1) )
-        return 0;
+        return IOREQ_STATUS_BAD;
 
     switch ( p->size )
     {


^ permalink raw reply	[flat|nested] 5+ messages in thread

end of thread, other threads:[~2024-09-23 10:39 UTC | newest]

Thread overview: 5+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
@@ -1175,7 +1175,7 @@ static int ioreq_send_buffered(struct io
 
     /*
-     * Return 0 for the cases we can't deal with:
+     * Return BAD for the cases we can't deal with:
      *  - 'addr' is only a 20-bit field, so we cannot address beyond 1MB
      *  - we cannot buffer accesses to guest memory buffers, as the guest
2024-09-11 12:19 [PATCH] ioreq: don't wrongly claim "success" in ioreq_send_buffered() Jan Beulich
2024-09-16 21:27 ` Julien Grall
2024-09-23  9:47   ` Jan Beulich
2024-09-23  9:55     ` Nicola Vetrini
2024-09-23 10:39       ` Jan Beulich

This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.